unit CCDCamera_Example;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, TeEngine, Series, TeeProcs, Chart, DataTypes, DllInterface,
  Mask, ImgList, inifiles, Buttons, ComCtrls, RzPanel;

const
  WM_USER_NOTIFY = WM_USER + 100;

  // For TCN1304, it has 3648 points
  MAX_CCD_POINTS = 3648;
  // For TCN1209, it only has 2048 points
  // MAX_CCD_POINTS = 2048;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    SetToNormalModeButton: TButton;
    SetToTriggerModeButton: TButton;
    SetExposureTimeButton: TButton;
    ExposureTimeEdit: TEdit;
    Label1: TLabel;
    GetCurrentFrameButton: TButton;
    StatusBar1: TStatusBar;
    Panel2: TPanel;
    ShowImageButton: TBitBtn;
    ShowFactoryPanelBitBtn: TBitBtn;
    ReSelectDeviceBitBtn: TBitBtn;
    DevicesComboBox: TComboBox;
    procedure FormShow(Sender: TObject);
    procedure DevicesComboBoxChange(Sender: TObject);
    procedure ShowFactoryPanelBitBtnClick(Sender: TObject);
    procedure ShowImageButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ReSelectDeviceBitBtnClick(Sender: TObject);
    procedure SetToNormalModeButtonClick(Sender: TObject);
    procedure SetToTriggerModeButtonClick(Sender: TObject);
    procedure SetExposureTimeButtonClick(Sender: TObject);
    procedure GetCurrentFrameButtonClick(Sender: TObject);
  private
    { Private declarations }
    CCDChartData : Array[0..MAX_CCD_POINTS] of integer;
    EngineStarted : Boolean;

    procedure DeviceReInitializeNeededMsg( var Msg : TMessage ); message WM_USER_NOTIFY;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  ImageCount : integer;

implementation

{$R *.DFM}

uses DeviceSelect;

procedure DeviceFaultCallBack( DevieType: integer ); cdecl;
begin
  {
    JTZ: DeviceFaultCallBack is invoked by camera engine while the device mapping is
    changed ( device plug/unplug ) OR device error occurred. In this case, the camera
    engine invokes this hooker, the engine will do the following before invoke tha
    callback:
    1). Internally, engine stops to grab frames from any device
    2). Engine will unhook this hooker...so this is a "One-Shot" hooker.
    In this case, camera engine is in "Sleep" state and can only be woke up by
    re-initialization of the devices.
    So application is supposed to do the following:
    1). Post a message for further operations, it's NOT recommended to do
    cleaning and re-initialization in this callback.
    2). In the message handler, it should do all the house keeping as following:
       CCDUSBInstallFrameHooker( 0, nil);
       CCDUSBInstallUSBDeviceHooker( nil);
       CCDUSBStopCameraEngine();
       CCDUSBUnInitDevice();
    3). And then, it's supposed to do ( User may also show a warning and close
        application):
       CCDUSBInitDevice();
       ...
  }
  PostMessage( MainForm.Handle, WM_USER_NOTIFY, 0 , 0 );
end;

procedure FrameCallBack( FrameType : integer; Row : integer ; Col : integer;
                         ImageProperty : PProcessedDataProperty; Buffer : PByte ); cdecl;
var
  i : integer;
  p : PWordArray;
begin
  {
    JTZ: For Linear camera, the callback is invoked in PanelFrom.Timer..which is in the
    main thread..so we're free to do any UI operations. Blocking the callback may pause
    the camera engine for a little while...the result is lower frame rate.
  }
  {
    In this sample code, we only process the frame from the current selected Device, note that
    camera engine will invoke frame callback for each Frame from any camera which was added
    in working set. In case of multiple cameras are used, the ImageProperty.CameraID is
    the identity of the camera (it's the same number userd for calling CCDUSBAddDeviceToWorkingSet()
  }
  if ImageProperty.CameraID = SelectedDevice then // This is the frame from current camera.
    begin
      Inc( ImageCount );
      MainForm.StatusBar1.Panels[2].Text := IntToStr( ImageProperty.CameraID );
      { ImageProperty.ExposureTime has the Exposure time for this frame }
      MainForm.StatusBar1.Panels[3].Text := IntToStr( ImageProperty.ExposureTime * 100 ) + ' us';
      { ImageProperty.TimeStamp has the TimeStamp (in 0.1ms unit) for this frame, note this
        is a 0 -- 65535 and round back number }
      MainForm.StatusBar1.Panels[4].Text := IntToStr( ImageProperty.TimeStamp );
      { ImageProperty.TriggerOccurred is mainly for NORMAL mode, while camera is in NORMAL
        mode, camera is still monitoring the TRIGGER signal and if it's asserted, the
        current frame will be marked with TriggerOccurred to be "1", otherwise it's "0",
        This gives application an alternative to poll the TRIGGER signal in NORMAL mode.
      }
      MainForm.StatusBar1.Panels[5].Text := IntToStr( ImageProperty.TriggerOccurred);
      {
        ImageProperty.TriggerEventCount is used for TRIGGER mode only, it's always "0" in
        NORMAL mode. While in TRIGGER mode, there might be multiple trigger signals (edges
        on Trigger Hardware Pin), each signal will increase this count by "1", and it's reset
        to "0" by application when setting the mode to "TRIGGER". (Note that application might
        set the mode to "TRIGGER" explicitly even when the current mode is already "TRIGGER",
        this setting is actually used for reset the count only).
        One thing worth to mention is that: Each frame takes more than 8ms, so if there's
        more than ONE signals in the frame time...the TriggerEventCount for frames might NOT
        be continuous, e.g. for this Frame, the count is 1, and next frame the count might
        be 8.
        In application, it's system integrator's responsibility to design the external trigger
        machenism properly.
      }
      MainForm.StatusBar1.Panels[6].Text := IntToStr( ImageProperty.TriggerEventCount);
      {
        While the image is over saturated, the frame data might be in-correct due to the residue
        of the electonics from the last frame...in this case, user should reduce the exposure time
        to correct this issue.
      }
      MainForm.StatusBar1.Panels[7].Text := IntToStr( ImageProperty.OverSaturated);
      p := PWordArray(Buffer);
      for i:= 0 to MAX_CCD_POINTS-1 do
        begin
          {
            It's up to user for processing these data.
          }
          MainForm.CCDChartData[i] :=  p[i];
        end;
    end;
end;

procedure TMainForm.DeviceReInitializeNeededMsg( var Msg : TMessage );
begin
  if TMessage(Msg).Msg = WM_USER_NOTIFY then
    begin
      // In this sample code, the "ReselectDevice" Button will do all the works
      // this message handler is supposed to do.
      ReSelectDeviceBitBtnClick( Self );
    end;
end;


procedure TMainForm.FormShow(Sender: TObject);
var
  device : integer;
  i : integer;
begin
  i:=0;
  EngineStarted := False;
  for device :=1 to CurrentDevices do
    begin
      {
        Note that CurrentDevices is the number of current connected CCD Camera
        Devices, but not all the them might be selected as "Working" camera,
        for example, we might have 5 CCD cameras connected on USB, but only
        2 and 4 are selected, in this case, we have:
        CameraSelected[] = (0, 1, 0, 1, 0);   2 and 4 are selected.
        Note that 2 and 4 are also the CameraID returned in the frame callback.
      }
      if CameraSelected[device] then
        begin
          CCDUSBAddDeviceToWorkingSet( device );
          DevicesComboBox.Items.Add( CameraSerialNo[device] );
          // The purpose of SelectableDevices is for quick getting of the
          // SelectedDevice ( SelectableDevices[DevicesComboBox.ItemIndex] )
          // Here the SelectedDevices is "1" based SELECTED device no.
          SelectableDevices[i] := device;
          Inc(i);
        end;
    end;
  if i = 0 then
    begin
      Close;
      Exit;
    end;

  // Let's install two callbacks here.
  CCDUSBInstallFrameHooker( 0, FrameCallBack );
  CCDUSBInstallUSBDeviceHooker( DeviceFaultCallBack );
  if  CCDUSBStartCameraEngine( Handle ) < 0 then
    begin
      Close;
      Exit;
    end;

  EngineStarted := True;
  DevicesComboBox.ItemIndex := 0;
  SelectedDevice := SelectableDevices[DevicesComboBox.ItemIndex];

  StatusBar1.Panels[0].Text := 'Selected: ' + IntToStr(CurrentSelectedDevices)
                               + ' Cameras';
  StatusBar1.Panels[1].Text := CameraModuleNo[SelectedDevice] + ':' +
                               CameraSerialNo[SelectedDevice];
  ShowImageButton.Caption := 'Start Grab';
end;

procedure TMainForm.DevicesComboBoxChange(Sender: TObject);
begin
  SelectedDevice := SelectableDevices[DevicesComboBox.ItemIndex];
  StatusBar1.Panels[1].Text := CameraModuleNo[SelectedDevice] + ':' +
                               CameraSerialNo[SelectedDevice];
end;



procedure TMainForm.ShowFactoryPanelBitBtnClick(Sender: TObject);
begin
  CCDUSBShowFactoryControlPanel( SelectedDevice, '661016' );
end;

procedure TMainForm.ShowImageButtonClick(Sender: TObject);
begin
  // Start or Stop Frame Grabbing.
  if ShowImageButton.Caption = 'Start Grab' then
    begin
      // Start the grabbing
      ShowImageButton.Caption := 'Stop Grab'; // Show "STOP" icon
      CCDUSBStartFrameGrab( FOREVER_TARGET_FRAMES );   // Get 10000 Frames only.
    end
  else // "STOP" icon is showing.
    begin
      // Stop the grabbing
      ShowImageButton.Caption := 'Start Grab'; // Show "START" icon.
      CCDUSBStopFrameGrab();
    end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // When user exit the application.
  if EngineStarted = True then
    CCDUSBStopCameraEngine();
  CCDUSBUnInitDevice();
end;

procedure TMainForm.ReSelectDeviceBitBtnClick(Sender: TObject);
var
  DevideForm : TDeviceForm;
begin
  // 1. House Keeping.
  CCDUSBInstallFrameHooker( 0, nil);
  CCDUSBInstallUSBDeviceHooker( nil);
  CCDUSBStopCameraEngine();
  CCDUSBUnInitDevice();
  DevicesComboBox.Items.Clear;
  DevicesComboBox.ItemIndex := -1;
  
  // 2. Show Device Selection Form again
  DeviceForm := TDeviceForm.Create( Application );
  DeviceForm.ShowModal;
  DeviceForm.Free;

  // 3. Explicitly call FormShow();
  // In FormShow(), it will install hookers and start the camera engine.
  FormShow( Self );
end;

procedure TMainForm.SetToNormalModeButtonClick(Sender: TObject);
begin
  CCDUSBSetCameraWorkMode( SelectedDevice, 0 );
end;

procedure TMainForm.SetToTriggerModeButtonClick(Sender: TObject);
begin
    CCDUSBSetCameraWorkMode( SelectedDevice, 1 );
end;

procedure TMainForm.SetExposureTimeButtonClick(Sender: TObject);
var
  ExposureTime : integer;
begin
  ExposureTime := StrToInt( ExposureTimeEdit.Text );
  CCDUSBSetExposureTime( SelectedDevice, ExposureTime, False );
end;

procedure TMainForm.GetCurrentFrameButtonClick(Sender: TObject);
var
  MyArray : Array[0..4095] of Word;
  wordPtr, res : PWord;
  i : integer;
begin
  res := CCDUSBGetCurrentFrame(SelectedDevice, wordPtr ) ;
  if res <> nil then
    begin
      for i:=0 to (8+3648-1) do // it's (8+2048-1) for TCN-1209.
        begin
          MyArray[i] := wordPtr^;
          Inc(wordPtr);
        end;
    end;
end;

end.
